@@ -8,14 +8,16 @@ from collections.abc import MutableMapping as CMutableMapping
 from collections.abc import MutableSequence as CMutableSequence
 from collections.abc import Sequence as CSequence
 from types import ModuleType
+from typing import ForwardRef  # type: ignore[import-error]  # pytype (for ForwardRef)
 from typing import (
     TYPE_CHECKING,
+    Any,
     Dict,
-    ForwardRef,
     List,
     Mapping,
     MutableMapping,
     MutableSequence,
+    NamedTuple,
     NoReturn,
     Optional,
     Sequence,
@@ -24,10 +26,11 @@ from typing import (
     TypeVar,
     Union,
 )
-from typing import _eval_type as eval_type  # type: ignore  # private API not in stubs
-from typing import _type_check as type_check  # type: ignore  # private API not in stubs
+from typing import _eval_type as eval_type  # type: ignore[attr-defined]
+from typing import _type_check as type_check  # type: ignore[attr-defined]
 from typing import cast, overload
 
+# get_type_hints
 try:
     # Python 3.7+
     from typing_extensions import get_type_hints  # type: ignore[attr-defined]
@@ -49,33 +52,46 @@ else:
                     pass
 
 
+# TypeGuard
+if sys.version_info >= (3, 10):
+    from typing import TypeGuard  # Python 3.10+
+else:
+    try:
+        from typing_extensions import (
+            TypeGuard,  # type: ignore[not-supported-yet]  # pytype
+        )
+    except ImportError:
+        if not TYPE_CHECKING:
+
+            class TypeGuard:
+                def __class_getitem__(cls, key):
+                    return bool
+
+
 # get_origin, get_args
 if sys.version_info >= (3, 8):
     from typing import get_args, get_origin  # Python 3.8+
 
 elif sys.version_info >= (3, 7):
-    from typing import _GenericAlias  # type: ignore  # private API not in stubs
+    from typing import _GenericAlias  # type: ignore[attr-defined]
 
     def get_origin(tp: object) -> Optional[object]:
-        if isinstance(tp, _GenericAlias):
-            return tp.__origin__  # type: ignore  # private API not in stubs
+        if isinstance(tp, _GenericAlias):  # type: ignore[16]  # pyre
+            return tp.__origin__  # type: ignore[reportGeneralTypeIssues]  # pyright
         else:
             return None
 
     def get_args(tp: object) -> Tuple[object, ...]:
-        if isinstance(tp, _GenericAlias):
-            return tp.__args__  # type: ignore  # private API not in stubs
+        if isinstance(tp, _GenericAlias):  # type: ignore[16]  # pyre
+            return tp.__args__  # type: ignore[reportGeneralTypeIssues]  # pyright
         else:
             return ()
 
 elif sys.version_info >= (3, 6):
-    from typing import GenericMeta  # type: ignore  # private API not in stubs
-    from typing import _Union  # type: ignore  # private API not in stubs
+    from typing import GenericMeta, _Union
 
     try:
-        from typing_extensions import (  # type: ignore  # private API not in stubs
-            _Literal,
-        )
+        from typing_extensions import _Literal
     except ImportError:
         if not TYPE_CHECKING:
 
@@ -84,7 +100,7 @@ elif sys.version_info >= (3, 6):
 
     def get_origin(tp: object) -> Optional[object]:
         if isinstance(tp, GenericMeta):
-            return tp.__origin__  # type: ignore  # private API not in stubs
+            return tp.__origin__
         elif isinstance(tp, _Union):
             return Union
         elif isinstance(tp, _Literal):
@@ -94,7 +110,7 @@ elif sys.version_info >= (3, 6):
 
     def get_args(tp: object) -> Tuple[object, ...]:
         if isinstance(tp, GenericMeta):
-            return tp.__args__  # type: ignore  # private API not in stubs
+            return tp.__args__
         elif isinstance(tp, _Union):
             return tp.__args__
         elif isinstance(tp, _Literal):
@@ -109,29 +125,29 @@ else:
 # _is_typed_dict
 _typed_dict_meta_list = []
 try:
-    # private API not in stubs.
-    from typing import _TypedDictMeta as _TypingTypedDictMeta  # type: ignore
+    from typing import (  # type: ignore[attr-defined]  # isort: skip
+        _TypedDictMeta as _TypingTypedDictMeta,  # type: ignore[reportGeneralTypeIssues]  # pyright
+    )
 
-    _typed_dict_meta_list.append(_TypingTypedDictMeta)
+    _typed_dict_meta_list.append(_TypingTypedDictMeta)  # type: ignore[16]  # pyre
 except ImportError:
     pass
 
 try:
-    from typing_extensions import (  # type: ignore # isort: skip
-        _TypedDictMeta as _TypingExtensionsTypedDictMeta,
+    from typing_extensions import (  # type: ignore[attr-defined]  # isort: skip
+        _TypedDictMeta as _TypingExtensionsTypedDictMeta,  # type: ignore[reportGeneralTypeIssues]  # pyright
     )
 
-    _typed_dict_meta_list.append(_TypingExtensionsTypedDictMeta)
+    _typed_dict_meta_list.append(_TypingExtensionsTypedDictMeta)  # type: ignore[16]  # pyre
 except ImportError:
     pass
 
-
 try:
-    from mypy_extensions import (  # type: ignore # isort: skip
-        _TypedDictMeta as _MypyExtensionsTypedDictMeta,
+    from mypy_extensions import (  # type: ignore[attr-defined]  # isort: skip
+        _TypedDictMeta as _MypyExtensionsTypedDictMeta,  # type: ignore[reportGeneralTypeIssues]  # pyright
     )
 
-    _typed_dict_meta_list.append(_MypyExtensionsTypedDictMeta)
+    _typed_dict_meta_list.append(_MypyExtensionsTypedDictMeta)  # type: ignore[16]  # pyre
 except ImportError:
     pass
 
@@ -142,67 +158,68 @@ def _is_typed_dict(tp: object) -> bool:
     return isinstance(tp, _typed_dict_metas)
 
 
-__all__ = ("trycast",)
+__all__ = (
+    "trycast",
+    "isassignable",
+    # NOTE: May be part of the API in the future
+    # "eval_type_str",
+)
 
 
 _T = TypeVar("_T")
 _F = TypeVar("_F")
 _SimpleTypeVar = TypeVar("_SimpleTypeVar")
-_SimpleTypeVarCo = TypeVar("_SimpleTypeVarCo", covariant=True)
+_SimpleTypeVarCo = TypeVar("_SimpleTypeVarCo", covariant=True)  # type: ignore[not-supported-yet]  # pytest
 
 _MISSING = object()
 _FAILURE = object()
 
-if TYPE_CHECKING:
-    from typing_extensions import TypeAlias, TypeGuard
-
-    TypeGuard_T: TypeAlias = TypeGuard[_T]
-else:
-    TypeGuard_T = bool
-
 # ------------------------------------------------------------------------------
 # trycast
 
-# TODO: Use this signature for trycast once support for TypeForm is
-#       implemented in mypy. See: https://github.com/python/mypy/issues/9773
+# TODO: Once support for TypeForm is implemented in mypy,
+#       replace the   `(Type[T]) -> Optional[T]` overload
+#       and the       `(object) -> Optional[object]` overload with
+#       the following `(TypeForm[T]) -> Optional[T]` overload:
+#
+#       See: https://github.com/python/mypy/issues/9773
 # @overload
 # def trycast(tp: TypeForm[_T], value: object) -> Optional[_T]: ...
-# @overload
-# def trycast(tp: TypeForm[_T], value: object, failure: _F) -> Union[_T, _F]: ...
 
 
 @overload
-def trycast(
-    tp: Type[_T], value: object, *, strict: bool = False, eval: bool = True
-) -> Optional[_T]:
-    ...
-
-
-# TODO: Replace `tp: str` with `tp: LiteralString` once support for
-#       LiteralString is generally available.
-@overload
-def trycast(
-    tp: str, value: object, *, strict: bool = False, eval: Literal[False]
+def trycast(  # type: ignore[43]  # pyre
+    tp: str, value: object, *, strict: bool = True, eval: Literal[False]
 ) -> NoReturn:
     ...
 
 
 @overload
-def trycast(
-    tp: object, value: object, *, strict: bool = False, eval: bool = True
+def trycast(tp: str, value: object, *, strict: bool = True, eval: bool = True) -> bool:  # type: ignore[43]  # pyre
+    ...
+
+
+@overload
+def trycast(  # type: ignore[43]  # pyre
+    tp: Type[_T], value: object, *, strict: bool = True, eval: bool = True
+) -> Optional[_T]:
+    ...
+
+
+@overload
+def trycast(  # type: ignore[43]  # pyre
+    tp: object, value: object, *, strict: bool = True, eval: bool = True
 ) -> Optional[object]:
     ...
 
 
-# TODO: Replace `tp: str` with `tp: LiteralString` once support for
-#       LiteralString is generally available.
 @overload
 def trycast(
     tp: str,
     value: object,
-    failure: _F,
+    failure: object,
     *,
-    strict: bool = False,
+    strict: bool = True,
     eval: Literal[False],
 ) -> NoReturn:
     ...
@@ -210,19 +227,19 @@ def trycast(
 
 @overload
 def trycast(
-    tp: Type[_T], value: object, failure: _F, *, strict: bool = False, eval: bool = True
+    tp: Type[_T], value: object, failure: _F, *, strict: bool = True, eval: bool = True
 ) -> Union[_T, _F]:
     ...
 
 
 @overload
 def trycast(
-    tp: object, value: object, failure: _F, *, strict: bool = False, eval: bool = True
+    tp: object, value: object, failure: _F, *, strict: bool = True, eval: bool = True
 ) -> Union[object, _F]:
     ...
 
 
-def trycast(tp, value, failure=None, *, strict=False, eval=True):
+def trycast(tp, value, failure=None, *, strict=True, eval=True):
     """
     If `value` is in the shape of `tp` (as accepted by a Python typechecker
     conforming to PEP 484 "Type Hints") then returns it, otherwise returns
@@ -272,9 +289,10 @@ def trycast(tp, value, failure=None, *, strict=False, eval=True):
     * UnresolvableTypeError --
         If `tp` is a string that could not be resolved to a type.
     """
+    options = _TrycastOptions(strict, eval)
     if isinstance(tp, str):
-        if eval:
-            tp = eval_type_str(tp)
+        if eval:  # == options.eval (for pytype)
+            tp = eval_type_str(tp)  # does use eval()
         else:
             raise UnresolvableTypeError(
                 f"Could not resolve type {tp!r}: "
@@ -283,18 +301,41 @@ def trycast(tp, value, failure=None, *, strict=False, eval=True):
                 f"disabling eval of string type references."
             )
     else:
-        tp = type_check(tp, "trycast() requires a type as its first argument.")
+        try:
+            tp = type_check(tp, "trycast() requires a type as its first argument.")
+        except TypeError:
+            if isinstance(tp, tuple) and len(tp) >= 1 and isinstance(tp[0], type):
+                raise TypeError(
+                    "trycast does not support checking against a tuple of types. "
+                    "Try checking against a Union[T1, T2, ...] instead."
+                )
+            else:
+                raise
     try:
-        return _trycast_inner(tp, value, failure, strict)
+        return _trycast_inner(tp, value, failure, options)  # type: ignore[bad-return-type]  # pytype
     except UnresolvedForwardRefError:
+        if options.eval:
+            advise = (
+                "Try altering the first type argument to be a string "
+                "reference (surrounded with quotes) instead."
+            )
+        else:
+            advise = (
+                "trycast() cannot resolve string type references "
+                "because it was called with eval=False."
+            )
         raise UnresolvedForwardRefError(
             f"trycast does not support checking against type form {tp!r} "
             "which contains a string-based forward reference. "
-            "Try altering the first type argument to be a string "
-            "reference (surrounded with quotes) instead."
+            f"{advise}"
         )
 
 
+class _TrycastOptions(NamedTuple):
+    strict: bool
+    eval: bool
+
+
 # TODO: Use this signature for _trycast_inner once support for TypeForm is
 #       implemented in mypy. See: https://github.com/python/mypy/issues/9773
 # @overload
@@ -303,19 +344,19 @@ def trycast(tp, value, failure=None, *, strict=False, eval=True):
 
 @overload
 def _trycast_inner(
-    tp: Type[_T], value: object, failure: _F, strict: bool
+    tp: Type[_T], value: object, failure: _F, options: _TrycastOptions
 ) -> Union[_T, _F]:
     ...
 
 
 @overload
 def _trycast_inner(
-    tp: object, value: object, failure: _F, strict: bool
+    tp: object, value: object, failure: _F, options: _TrycastOptions
 ) -> Union[object, _F]:
     ...
 
 
-def _trycast_inner(tp, value, failure, strict):
+def _trycast_inner(tp, value, failure, options):
     """
     Raises:
     * TypeNotSupportedError
@@ -340,7 +381,7 @@ def _trycast_inner(tp, value, failure, strict):
 
     type_origin = get_origin(tp)
     if type_origin is list or type_origin is List:  # List, List[T]
-        return _trycast_listlike(tp, value, failure, list, strict)
+        return _trycast_listlike(tp, value, failure, list, options)
 
     if type_origin is tuple or type_origin is Tuple:
         if isinstance(value, tuple):
@@ -355,7 +396,7 @@ def _trycast_inner(tp, value, failure, strict):
                     value,
                     failure,
                     tuple,
-                    strict,
+                    options,
                     covariant_t=True,
                     t_ellipsis=True,
                 )
@@ -364,7 +405,7 @@ def _trycast_inner(tp, value, failure, strict):
                     return failure
 
                 for (T, t) in zip(type_args, value):
-                    if _trycast_inner(T, t, _FAILURE, strict) is _FAILURE:
+                    if _trycast_inner(T, t, _FAILURE, options) is _FAILURE:
                         return failure
 
                 return cast(_T, value)
@@ -373,86 +414,104 @@ def _trycast_inner(tp, value, failure, strict):
 
     if type_origin is Sequence or type_origin is CSequence:  # Sequence, Sequence[T]
         return _trycast_listlike(
-            tp, value, failure, CSequence, strict, covariant_t=True
+            tp, value, failure, CSequence, options, covariant_t=True
         )
 
     if (
         type_origin is MutableSequence or type_origin is CMutableSequence
     ):  # MutableSequence, MutableSequence[T]
-        return _trycast_listlike(tp, value, failure, CMutableSequence, strict)
+        return _trycast_listlike(tp, value, failure, CMutableSequence, options)
 
     if type_origin is dict or type_origin is Dict:  # Dict, Dict[K, V]
-        return _trycast_dictlike(tp, value, failure, dict, strict)
+        return _trycast_dictlike(tp, value, failure, dict, options)
 
     if type_origin is Mapping or type_origin is CMapping:  # Mapping, Mapping[K, V]
-        return _trycast_dictlike(tp, value, failure, CMapping, strict, covariant_v=True)
+        return _trycast_dictlike(
+            tp, value, failure, CMapping, options, covariant_v=True
+        )
 
     if (
         type_origin is MutableMapping or type_origin is CMutableMapping
     ):  # MutableMapping, MutableMapping[K, V]
-        return _trycast_dictlike(tp, value, failure, CMutableMapping, strict)
+        return _trycast_dictlike(tp, value, failure, CMutableMapping, options)
 
     if type_origin is Union:  # Union[T1, T2, ...]
         for T in get_args(tp):
-            if _trycast_inner(T, value, _FAILURE, strict) is not _FAILURE:
-                return cast(_T, value)
+            if _trycast_inner(T, value, _FAILURE, options) is not _FAILURE:  # type: ignore[wrong-arg-types]  # pytype
+                if isinstance(tp, type):
+                    return cast(_T, value)
+                else:
+                    return value
         return failure
 
     if type_origin is Literal:  # Literal[...]
         for literal in get_args(tp):
             if value == literal:
-                return cast(_T, value)
+                if isinstance(tp, type):
+                    return cast(_T, value)
+                else:
+                    return value
         return failure
 
     if _is_typed_dict(tp):  # T extends TypedDict
         if isinstance(value, Mapping):
-            resolved_annotations = get_type_hints(
-                tp
-            )  # resolve ForwardRefs in tp.__annotations__
+            if options.eval:
+                resolved_annotations = get_type_hints(  # does use eval()
+                    tp
+                )  # resolve ForwardRefs in tp.__annotations__
+            else:
+                resolved_annotations = tp.__annotations__  # type: ignore[attribute-error]  # pytype
 
             try:
                 # {typing in Python 3.9+, typing_extensions}.TypedDict
-                required_keys = tp.__required_keys__
+                required_keys = tp.__required_keys__  # type: ignore[attribute-error]  # pytype
             except AttributeError:
                 # {typing in Python 3.8, mypy_extensions}.TypedDict
-                if strict:
+                if options.strict:
                     if sys.version_info[:2] >= (3, 9):
                         advise = "Suggest use a typing.TypedDict instead."
                     else:
                         advise = "Suggest use a typing_extensions.TypedDict instead."
+                    advise2 = "Or use trycast(..., strict=False)."
                     raise TypeNotSupportedError(
-                        "trycast cannot determine which keys are required "
-                        "and which are potentially-missing for the "
-                        "specified kind of TypedDict. " + advise
+                        f"trycast cannot determine which keys are required "
+                        f"and which are potentially-missing for the "
+                        f"specified kind of TypedDict. {advise} {advise2}"
                     )
                 else:
-                    if tp.__total__:
+                    if tp.__total__:  # type: ignore[attribute-error]  # pytype
                         required_keys = resolved_annotations.keys()
                     else:
                         required_keys = frozenset()
 
-            for (k, v) in value.items():
+            for (k, v) in value.items():  # type: ignore[attribute-error]  # pytype
                 V = resolved_annotations.get(k, _MISSING)
-                if V is _MISSING or _trycast_inner(V, v, _FAILURE, strict) is _FAILURE:
+                if V is _MISSING or _trycast_inner(V, v, _FAILURE, options) is _FAILURE:
                     return failure
 
             for k in required_keys:
-                if k not in value:
+                if k not in value:  # type: ignore[unsupported-operands]  # pytype
                     return failure
-            return cast(_T, value)
+            if isinstance(tp, type):
+                return cast(_T, value)
+            else:
+                return value
         else:
             return failure
 
+    if tp is Any:
+        if isinstance(tp, type):
+            return cast(_T, value)
+        else:
+            return value
+
+    if tp is NoReturn:
+        return failure
+
     if isinstance(tp, ForwardRef):
         raise UnresolvedForwardRefError()
 
-    if isinstance(tp, tuple):
-        raise TypeError(
-            "trycast does not support checking against a tuple of types. "
-            "Try checking against a Union[T1, T2, ...] instead."
-        )
-
-    if isinstance(value, tp):  # type: ignore
+    if isinstance(value, tp):  # type: ignore[wrong-arg-types]  # pytype
         return value
     else:
         return failure
@@ -472,10 +531,10 @@ def _trycast_listlike(
     value: object,
     failure: _F,
     listlike_type: Type,
-    strict: bool,
+    options: _TrycastOptions,
     *,
-    covariant_t: bool,
-    t_ellipsis: bool,
+    covariant_t: bool = False,
+    t_ellipsis: bool = False,
 ) -> Union[_T, _F]:
     ...
 
@@ -486,16 +545,16 @@ def _trycast_listlike(
     value: object,
     failure: _F,
     listlike_type: Type,
-    strict: bool,
+    options: _TrycastOptions,
     *,
-    covariant_t: bool,
-    t_ellipsis: bool,
+    covariant_t: bool = False,
+    t_ellipsis: bool = False,
 ) -> Union[object, _F]:
     ...
 
 
 def _trycast_listlike(
-    tp, value, failure, listlike_type, strict, *, covariant_t=False, t_ellipsis=False
+    tp, value, failure, listlike_type, options, *, covariant_t=False, t_ellipsis=False
 ):
     if isinstance(value, listlike_type):
         T_ = get_args(tp)
@@ -515,11 +574,14 @@ def _trycast_listlike(
         if _is_simple_typevar(T, covariant=covariant_t):
             pass
         else:
-            for x in value:
-                if _trycast_inner(T, x, _FAILURE, strict) is _FAILURE:
+            for x in value:  # type: ignore[attribute-error]  # pytype
+                if _trycast_inner(T, x, _FAILURE, options) is _FAILURE:
                     return failure
 
-        return cast(_T, value)
+        if isinstance(tp, type):
+            return cast(_T, value)
+        else:
+            return value
     else:
         return failure
 
@@ -530,9 +592,9 @@ def _trycast_dictlike(
     value: object,
     failure: _F,
     dictlike_type: Type,
-    strict: bool,
+    options: _TrycastOptions,
     *,
-    covariant_t: bool,
+    covariant_v: bool = False,
 ) -> Union[_T, _F]:
     ...
 
@@ -543,14 +605,14 @@ def _trycast_dictlike(
     value: object,
     failure: _F,
     dictlike_type: Type,
-    strict: bool,
+    options: _TrycastOptions,
     *,
-    covariant_t: bool,
+    covariant_v: bool = False,
 ) -> Union[object, _F]:
     ...
 
 
-def _trycast_dictlike(tp, value, failure, dictlike_type, strict, *, covariant_v=False):
+def _trycast_dictlike(tp, value, failure, dictlike_type, options, *, covariant_v=False):
     if isinstance(value, dictlike_type):
         K_V = get_args(tp)
 
@@ -565,45 +627,55 @@ def _trycast_dictlike(tp, value, failure, dictlike_type, strict, *, covariant_v=
         if _is_simple_typevar(K) and _is_simple_typevar(V, covariant=covariant_v):
             pass
         else:
-            for (k, v) in value.items():
+            for (k, v) in value.items():  # type: ignore[attribute-error]  # pytype
                 if (
-                    _trycast_inner(K, k, _FAILURE, strict) is _FAILURE
-                    or _trycast_inner(V, v, _FAILURE, strict) is _FAILURE
+                    _trycast_inner(K, k, _FAILURE, options) is _FAILURE
+                    or _trycast_inner(V, v, _FAILURE, options) is _FAILURE
                 ):
                     return failure
-        return cast(_T, value)
+        if isinstance(tp, type):
+            return cast(_T, value)
+        else:
+            return value
     else:
         return failure
 
 
 def _is_simple_typevar(T: object, covariant: bool = False) -> bool:
     return (
-        isinstance(T, TypeVar)
-        and T.__constraints__ == ()
-        and T.__covariant__ == covariant
-        and T.__contravariant__ is False
-        and T.__constraints__ == ()
+        isinstance(T, TypeVar)  # type: ignore[wrong-arg-types]  # pytype
+        and T.__constraints__ == ()  # type: ignore[attribute-error]  # pytype
+        and T.__covariant__ == covariant  # type: ignore[attribute-error]  # pytype
+        and T.__contravariant__ is False  # type: ignore[attribute-error]  # pytype
+        and T.__constraints__ == ()  # type: ignore[attribute-error]  # pytype
     )
 
 
 # ------------------------------------------------------------------------------
 # isassignable
 
-# TODO: Use this signature for isassignable once support for TypeForm is
-#       implemented in mypy. See: https://github.com/python/mypy/issues/9773
+# TODO: Once support for TypeForm is implemented in mypy,
+#       replace the   `(Type[T]) -> TypeGuard[T]` overload
+#       and the       `(object) -> bool` overload with
+#       the following `(TypeForm[T]) -> TypeGuard[T]` overload:
+#
+#       See: https://github.com/python/mypy/issues/9773
 # @overload
-# def isassignable(value: object, tp: TypeForm[_T]) -> TypeGuard_T: ...
+# def isassignable(value: object, tp: TypeForm[_T]) -> TypeGuard[_T]: ...
 
 
-# TODO: Replace `tp: str` with `tp: LiteralString` once support for
-#       LiteralString is generally available.
 @overload
 def isassignable(value: object, tp: str, *, eval: Literal[False]) -> NoReturn:
     ...
 
 
 @overload
-def isassignable(value: object, tp: Type[_T], *, eval: bool = True) -> TypeGuard_T:
+def isassignable(value: object, tp: str, *, eval: bool = True) -> bool:
+    ...
+
+
+@overload
+def isassignable(value: object, tp: Type[_T], *, eval: bool = True) -> TypeGuard[_T]:  # type: ignore[invalid-annotation]  # pytype
     ...
 
 
@@ -612,7 +684,7 @@ def isassignable(value: object, tp: object, *, eval: bool = True) -> bool:
     ...
 
 
-def isassignable(value, tp, *, eval: bool = True):
+def isassignable(value, tp, *, eval=True):
     """
     Returns whether `value` is in the shape of `tp`
     (as accepted by a Python typechecker conforming to PEP 484 "Type Hints").
@@ -654,10 +726,19 @@ def isassignable(value, tp, *, eval: bool = True):
     * UnresolvableTypeError --
         If `tp` is a string that could not be resolved to a type.
     """
-    return (
-        trycast(tp, value, _isassignable_failure, strict=True, eval=eval)
-        is not _isassignable_failure
-    )
+    if isinstance(tp, type):
+        return cast(  # type: ignore[invalid-annotation]  # pytype
+            TypeGuard[_T],  # type: ignore[not-indexable]  # pytype
+            (
+                trycast(tp, value, _isassignable_failure, strict=True, eval=eval)
+                is not _isassignable_failure
+            ),
+        )
+    else:
+        return (
+            trycast(tp, value, _isassignable_failure, strict=True, eval=eval)
+            is not _isassignable_failure
+        )
 
 
 _isassignable_failure = object()
@@ -668,7 +749,7 @@ _isassignable_failure = object()
 
 _IMPORTABLE_TYPE_EXPRESSION_RE = re.compile(r"^((?:[a-zA-Z0-9_]+\.)+)(.*)$")
 _UNIMPORTABLE_TYPE_EXPRESSION_RE = re.compile(r"^[a-zA-Z0-9_]+(\[.*\])?$")
-_BUILTINS_MODULE = builtins
+_BUILTINS_MODULE: ModuleType = builtins
 _EXTRA_ADVISE_IF_MOD_IS_BUILTINS = (
     " Try altering the type argument to be a string "
     "reference (surrounded with quotes) instead, "
@@ -712,7 +793,7 @@ def eval_type_str(tp: str) -> object:
         module_name = module_name_dot[:-1]
         try:
             mod = importlib.import_module(module_name)
-        except:  # noqa: E722
+        except Exception:
             raise UnresolvableTypeError(
                 f"Could not resolve type {tp!r}: " f"Could not import {module_name!r}."
             )
@@ -731,7 +812,7 @@ def eval_type_str(tp: str) -> object:
     # Lookup the type from a module
     try:
         member = eval(member_expr, mod.__dict__, None)
-    except:  # noqa: E722
+    except Exception:
         raise UnresolvableTypeError(
             f"Could not resolve type {tp!r}: "
             f"Could not eval {member_expr!r} inside module {module_name!r}."
@@ -744,8 +825,8 @@ def eval_type_str(tp: str) -> object:
 
     # Resolve any ForwardRef instances inside the type
     try:
-        member = eval_type(member, mod.__dict__, None)
-    except:  # noqa: E722
+        member = eval_type(member, mod.__dict__, None)  # type: ignore[16]  # pyre
+    except Exception:
         raise UnresolvableTypeError(
             f"Could not resolve type {tp!r}: "
             f"Could not eval type {member!r} inside module {module_name!r}."
@@ -755,7 +836,7 @@ def eval_type_str(tp: str) -> object:
     # 1. Ensure the object is actually a type
     # 2. As a special case, interpret None as type(None)
     try:
-        member = type_check(member, f"Could not resolve type {tp!r}: ")
+        member = type_check(member, f"Could not resolve type {tp!r}: ")  # type: ignore[16]  # pyre
     except TypeError as e:
         raise UnresolvableTypeError(str(e))
     return member
